# The intercom security policy.
# -----------------------------
#
# This security policy enables a Tclet to:
#
# * Discover the names of other Tclets using this policy.
# * Send messages to all or some of the other Tclets now using
#   this policy.
# * Receive messages from other Tclets using the policy.
# * Register a callback that will be called when another Tclet sends
#   a message.
#
# Permissions to use this policy are controlled by settings in the file
# trust.map in the ../trust directory.
#
# NOTE: This is very likely to be repackaged as a feature instead of a
#	policy, soon.
#
# CONTACT:		sunscript-plugin@sunscript.sun.com
#
# AUTHORS:		Jacob Levy			Laurent Demailly
#			jyl@eng.sun.com			demailly@eng.sun.com
#			jyl@tcl-tk.com			L@demailly.com
#
# Please contact us directly for questions, comments and enhancements.
#
# Copyright (c) 1996-1997 Sun Microsystems, Inc.
#
# See the file "license.terms" for information on usage and redistribution
# of this file, and for a DISCLAIMER OF ALL WARRANTIES.
#
# SCCS: @(#) intercom.tcl 1.9 97/09/26 07:21:38

# Install the policy:

proc intercom_PolicyInit {slave {arglist {}}} {
    global intercom_state		;# Policy state

    set intercom_state(user,$slave)	$slave
    set intercom_state($slave,queue)	{}
    set intercom_state($slave,callback)	{}

    lappend intercom_state(all) $slave

    # Add aliases for use of the policy:

    interp alias $slave intercom {} IntercomCommand $slave

    # Log the install of the policy:

    browser_log $slave security installed policy Intercom
}

# Clean up for this slave when the slave is destroyed.

proc intercom_PolicyCleanup {slave} {
    global intercom_state		;# Policy state

    unset intercom_state(user,$slave)
    set index [lsearch intercom_state(all) $slave]
    lreplace intercom_state(all) $index $index
    foreach i [array names intercom_state $slave,*] {
	unset intercom_state($i)
    }
}

# This procedure dispatches the different commands for the policy:

proc IntercomCommand {slave command args} {
    switch $command {
	send {IntercomSend $slave $args}
	receive {IntercomReceive $slave}
	callback {IntercomCallback $slave $args}
	names {IntercomNames}
    }
}

proc IntercomSend {slave arglist} {
    global intercom_state		;# Policy state

    set addressees ""
    if {"[lindex $arglist 0]" == "all"} {
	set addressees $intercom_state(all)
    } else {
	foreach a [lindex $arglist 0] {
	    if {[info exists intercom_state(user,$a)]} {
		lappend addressees $a
	    }
	}
    }

    # Send the message to each addressee. If the addressee has a registered
    # callback, invoke it, otherwise just enqueue the message.

    set msg [list [expr abs([clock clicks])] $slave [lindex $arglist 1]]
    foreach a $addressees {
	if {"$intercom_state($a,callback)" != ""} {
	    interp eval $a [list $intercom_state($a,callback) $msg]
	} else {
	    lappend intercom_state($a,queue) $msg
	}
    }
}

proc IntercomReceive {slave} {
    global intercom_state		;# Policy state

    if {"$intercom_state($slave,queue)" == ""} {
	return
    }
    set msg [lindex $intercom_state($slave,queue) 0]
    set intercom_state($slave,queue) \
	[lrange $intercom_state($slave,queue) 1 end]
    return $msg
}

proc IntercomCallback {slave args} {
    global intercom_state		;# Policy state

    if {"$args" == ""} {
	return $intercom_state($slave,callback)
    }
    set old_callback $intercom_state($slave,callback)
    set intercom_state($slave,callback) [lindex $args 0]
    return $old_callback
}

proc IntercomNames {} {
    global intercom_state		;# Policy state

    set users ""
    foreach u [array names intercom_state user,*] {
	lappend users $intercom_state($u)
    }
    return $users
}


